/** \file
*/

#include "adc.h"
#include "board.h"

// ADC field definition for the Mode Register
/* Reminder            TRGEN    => Selection bewteen Software or hardware start of conversion
                       TRGSEL   => Relevant if the previous field set a Hardware Triggering Mode
                 		0x0 << 1 (ADC) Selected TRGSEL = TIAO0
				0x1 << 1 (ADC) Selected TRGSEL = TIAO1
				0x2 << 1 (ADC) Selected TRGSEL = TIAO2
                       LOWRES   => 10-bit result if ths bit is cleared 0
                       SLEEP    => normal mode if ths is cleared
                       PRESCAL  => ADCclock = MCK / [(PRESCAL + 1)*2]
                       STARTUP  => Startup Time = [(STARTUP + 1)*8] / ADCclock
                       SHTIM    => Tracking time = (SHTIM + 1) / ADCclock
 */
#define   TRGEN    (0x1)    ///< Hardware triggering
#define   TRGSEL   (0x0)    ///< Use a Timer output signal (on rising edge) from TIOA0 (for this example)
#define   LOWRES   (0x1)    ///< 8-bit result output
#define   SLEEP    (0x0)    ///< Normal Mode

//#define   PRESCAL  (0x4)    // Max freq value for 10bit; MCK is divided by 10 (ADCClock = MCK / ( (PRESCAL+1) * 2 ))
#define   PRESCAL  (0x3)    ///< AD CLK = 6MHz (ADCClock = MCK / ( (PRESCAL+1) * 2 ))

/** \brief This time period must be higher than 20 s and not 20 ms;

	Startup Time = (STARTUP+1) * 8 / ADCClock
*/
#define   STARTUP  (0xf)    

/** \brief Sample & Hold time

	Must be higher than 3 ADC clock cycles but depends on output
	impedance of the analog driver to the ADC input.\n
	Sample & Hold Time = (SHTIM+1) / ADCClock
*/
#define   SHTIM    (0x2)    


/** Init A/D converter */
void ADCInit (void)
{
    /* Clear all previous setting and result */
    AT91F_ADC_SoftReset (AT91C_BASE_ADC);
    /* First step: Set the ADC by writing in Mode register */
    AT91F_ADC_CfgModeReg (AT91C_BASE_ADC, (SHTIM << 24) | (STARTUP << 16) | (PRESCAL << 8) |
                          (SLEEP << 5) | (LOWRES <<4) | (TRGSEL << 1) | (TRGEN )) ;
    /* Second Step: Select the active channel */
    AT91F_ADC_EnableChannel (AT91C_BASE_ADC, (1<<CHANNEL));	//ustawia ADC_CHER
}
